@@ -1,29 +1,25 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import absolute_import, unicode_literals
-
 import hashlib
+import mimetypes
 import os
+from datetime import datetime
 
 from django.conf import settings
-from django.core import urlresolvers
+from django.core.exceptions import ValidationError
 from django.core.files.base import ContentFile
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.urls import NoReverseMatch, reverse
+from django.utils import timezone
+from django.utils.functional import cached_property
+from django.utils.translation import gettext_lazy as _
+
+from polymorphic.managers import PolymorphicManager
+from polymorphic.models import PolymorphicModel
 
-from . import mixins
 from .. import settings as filer_settings
 from ..fields.multistorage_file import MultiStorageFileField
-from ..utils.compatibility import LTE_DJANGO_1_7, python_2_unicode_compatible
+from . import mixins
 from .foldermodels import Folder
 
-try:
-    from polymorphic.models import PolymorphicModel
-    from polymorphic.managers import PolymorphicManager
-except ImportError:
-    # django-polymorphic < 0.8
-    from polymorphic import PolymorphicModel, PolymorphicManager
-
 
 class FileManager(PolymorphicManager):
     def find_all_duplicates(self):
@@ -39,48 +35,114 @@ class FileManager(PolymorphicManager):
         return [i for i in self.exclude(pk=file_obj.pk).filter(sha1=file_obj.sha1)]
 
 
-@python_2_unicode_compatible
+def is_public_default():
+    # not using this setting directly as `is_public` default value
+    # so that Django doesn't generate new migrations upon setting change
+    return filer_settings.FILER_IS_PUBLIC_DEFAULT
+
+
+def mimetype_validator(value):
+    if not mimetypes.guess_extension(value):
+        msg = "'{mimetype}' is not a recognized MIME-Type."
+        raise ValidationError(msg.format(mimetype=value))
+
+
 class File(PolymorphicModel, mixins.IconsMixin):
     file_type = 'File'
     _icon = "file"
-    folder = models.ForeignKey(Folder, verbose_name=_('folder'), related_name='all_files',
-        null=True, blank=True)
+    _file_data_changed_hint = None
+
+    folder = models.ForeignKey(
+        Folder,
+        verbose_name=_('folder'),
+        related_name='all_files',
+        null=True,
+        blank=True,
+        on_delete=models.CASCADE,
+    )
     file = MultiStorageFileField(_('file'), null=True, blank=True, max_length=255)
-    _file_size = models.IntegerField(_('file size'), null=True, blank=True)
+    _file_size = models.BigIntegerField(_('file size'), null=True, blank=True)
 
     sha1 = models.CharField(_('sha1'), max_length=40, blank=True, default='')
 
     has_all_mandatory_data = models.BooleanField(_('has all mandatory data'), default=False, editable=False)
 
     original_filename = models.CharField(_('original filename'), max_length=255, blank=True, null=True)
-    name = models.CharField(max_length=255, default="", blank=True,
-        verbose_name=_('name'))
-    description = models.TextField(null=True, blank=True,
-        verbose_name=_('description'))
+    name = models.CharField(max_length=255, default="", blank=True, verbose_name=_('name'))
+    description = models.TextField(null=True, blank=True, verbose_name=_('description'))
 
-    owner = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
-        related_name='owned_%(class)ss', on_delete=models.SET_NULL,
-        null=True, blank=True, verbose_name=_('owner'))
+    owner = models.ForeignKey(
+        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
+        related_name='owned_%(class)ss',
+        on_delete=models.SET_NULL,
+        null=True,
+        blank=True,
+        verbose_name=_('owner'),
+    )
 
     uploaded_at = models.DateTimeField(_('uploaded at'), auto_now_add=True)
     modified_at = models.DateTimeField(_('modified at'), auto_now=True)
 
     is_public = models.BooleanField(
-        default=filer_settings.FILER_IS_PUBLIC_DEFAULT,
+        default=is_public_default,
         verbose_name=_('Permissions disabled'),
         help_text=_('Disable any permission checking for this '
                     'file. File will be publicly accessible '
                     'to anyone.'))
 
+    mime_type = models.CharField(
+        max_length=255,
+        help_text='MIME type of uploaded content',
+        validators=[mimetype_validator],
+        default='application/octet-stream',
+    )
+
     objects = FileManager()
 
     @classmethod
-    def matches_file_type(cls, iname, ifile, request):
+    def matches_file_type(cls, iname, ifile, mime_type):
         return True  # I match all files...
 
     def __init__(self, *args, **kwargs):
-        super(File, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._old_is_public = self.is_public
+        self.file_data_changed(post_init=True)
+
+    @cached_property
+    def mime_maintype(self):
+        return self.mime_type.split('/')[0]
+
+    @cached_property
+    def mime_subtype(self):
+        return self.mime_type.split('/')[1]
+
+    def file_data_changed(self, post_init=False):
+        """
+        This is called whenever self.file changes (including initial set in __init__).
+        MultiStorageFileField has a custom descriptor which calls this function when
+        field value is changed.
+        Returns True if data related attributes were updated, False otherwise.
+        """
+        if self._file_data_changed_hint is not None:
+            data_changed_hint = self._file_data_changed_hint
+            self._file_data_changed_hint = None
+            if not data_changed_hint:
+                return False
+        if post_init and self._file_size and self.sha1:
+            # When called from __init__, only update if values are empty.
+            # This makes sure that nothing is done when instantiated from db.
+            return False
+        # cache the file size
+        try:
+            self._file_size = self.file.size
+        except:   # noqa
+            self._file_size = None
+        # generate SHA1 hash
+        try:
+            self.generate_sha1()
+        except Exception:
+            self.sha1 = ''
+        return True
 
     def _move_file(self):
         """
@@ -106,9 +168,12 @@ class File(PolymorphicModel, mixins.IconsMixin):
         # This is needed because most of the remote File Storage backend do not
         # open the file.
         src_file = src_storage.open(src_file_name)
-        src_file.open()
-        self.file = dst_storage.save(dst_file_name,
-            ContentFile(src_file.read()))
+        # Context manager closes file after reading contents
+        with src_file.open() as f:
+            content_file = ContentFile(f.read())
+        # hint file_data_changed callback that data is actually unchanged
+        self._file_data_changed_hint = False
+        self.file = dst_storage.save(dst_file_name, content_file)
         src_storage.delete(src_file_name)
 
     def _copy_file(self, destination, overwrite=False):
@@ -153,27 +218,15 @@ class File(PolymorphicModel, mixins.IconsMixin):
             pass
         elif issubclass(self.__class__, File):
             self._file_type_plugin_name = self.__class__.__name__
-        # cache the file size
-        # TODO: only do this if needed (depending on the storage backend the whole file will be downloaded)
-        try:
-            self._file_size = self.file.size
-        except:
-            pass
         if self._old_is_public != self.is_public and self.pk:
             self._move_file()
             self._old_is_public = self.is_public
-        # generate SHA1 hash
-        # TODO: only do this if needed (depending on the storage backend the whole file will be downloaded)
-        try:
-            self.generate_sha1()
-        except Exception:
-            pass
-        super(File, self).save(*args, **kwargs)
+        super().save(*args, **kwargs)
     save.alters_data = True
 
     def delete(self, *args, **kwargs):
         # Delete the model before the file
-        super(File, self).delete(*args, **kwargs)
+        super().delete(*args, **kwargs)
         # Delete the file if there are no other Files referencing it.
         if not File.objects.filter(file=self.file.name, is_public=self.is_public).exists():
             self.file.delete(False)
@@ -206,7 +259,7 @@ class File(PolymorphicModel, mixins.IconsMixin):
         image. Return the string 'ALL' if the user has all rights.
         """
         user = request.user
-        if not user.is_authenticated():
+        if not user.is_authenticated:
             return False
         elif user.is_superuser:
             return True
@@ -225,12 +278,11 @@ class File(PolymorphicModel, mixins.IconsMixin):
         return text
 
     def get_admin_change_url(self):
-        if LTE_DJANGO_1_7:
-            model_name = self._meta.module_name
-        else:
-            model_name = self._meta.model_name
-        return urlresolvers.reverse(
-            'admin:{0}_{1}_change'.format(self._meta.app_label, model_name),
+        return reverse(
+            'admin:{0}_{1}_change'.format(
+                self._meta.app_label,
+                self._meta.model_name,
+            ),
             args=(self.pk,)
         )
 
@@ -241,7 +293,7 @@ class File(PolymorphicModel, mixins.IconsMixin):
         except AttributeError:
             # Django >1.6
             model_name = self._meta.model_name
-        return urlresolvers.reverse(
+        return reverse(
             'admin:{0}_{1}_delete'.format(self._meta.app_label, model_name,),
             args=(self.pk,))
 
@@ -252,20 +304,27 @@ class File(PolymorphicModel, mixins.IconsMixin):
         """
         try:
             r = self.file.url
-        except:
+        except:  # noqa
             r = ''
         return r
 
+    @property
+    def canonical_time(self):
+        if settings.USE_TZ:
+            return int((self.uploaded_at - datetime(1970, 1, 1, 1, tzinfo=timezone.utc)).total_seconds())
+        else:
+            return int((self.uploaded_at - datetime(1970, 1, 1, 1)).total_seconds())
+
     @property
     def canonical_url(self):
         url = ''
         if self.file and self.is_public:
             try:
-                url = urlresolvers.reverse('canonical', kwargs={
-                    'uploaded_at': self.uploaded_at.strftime('%s'),
+                url = reverse('canonical', kwargs={
+                    'uploaded_at': self.canonical_time,
                     'file_id': self.id
                 })
-            except urlresolvers.NoReverseMatch:
+            except NoReverseMatch:
                 pass  # No canonical url, return empty string
         return url
 
@@ -273,7 +332,7 @@ class File(PolymorphicModel, mixins.IconsMixin):
     def path(self):
         try:
             return self.file.path
-        except:
+        except:  # noqa
             return ""
 
     @property
@@ -315,7 +374,7 @@ class File(PolymorphicModel, mixins.IconsMixin):
     def duplicates(self):
         return File.objects.find_duplicates(self)
 
-    class Meta(object):
+    class Meta:
         app_label = 'filer'
         verbose_name = _('file')
         verbose_name_plural = _('files')
